import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import processing.sound.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class prote extends PApplet {



SoundFile bulletSound;
SoundFile absorbSound;
SoundFile tristam;

BulletManager bullets;
ParticleManager particles;

GSM gsm;

PFont font;
PFont font2;

int currentDay = 0;

int shakeIntensity = 5;
float zoomIntensity = 1.2f;
int bulletTimer;
int t;

int enemyKilled;

PImage light;

boolean isShaking = !true;
boolean isZooming = !true;
int shakeTimer;
int zoomTimer;

public void setup() {
  
  colorMode(HSB);
  gsm = new GSM();
  gsm.setState(new SplashState());
  //gsm.setState(new PlayState(0));

  font = createFont("verdanab.ttf", 64);
  font2 = createFont("verdanab.ttf", 32);
  textAlign(CENTER, CENTER);
  textFont(font);

  bullets = new BulletManager();
  particles = new ParticleManager();

  noStroke();

  light = loadImage("light.png");
  
  bulletSound = new SoundFile(this, "Powerup245.wav");
  absorbSound = new SoundFile(this, "absorb.wav");
  tristam = new SoundFile(this,"bgsound.mp3");
  tristam.loop();
}

public void draw() {
  pushMatrix();
  if (isShaking) {
    translate(random(shakeIntensity), random(shakeIntensity));
  }
  if (isZooming) {
    translate(0, 0);
    scale(map(sin(t/5f), -1, 1, 1, 1.01f), map(sin(t/5f), -1, 1, 1, 1.01f));
  }
  gsm.update();
  gsm.render();

  bullets.render();
  particles.render();

  popMatrix();

  if (shakeTimer-- <= 0) {
    isShaking = false;
    shakeIntensity = 5;
  }

  t++;
}
class Bullet extends Sprite{
  Bullet(int x,int y,int r,int c){
    super(x,y,r,c);
    alive = !true;
  }
  
  public void create(int x,int y,int vx,int vy,int r){
    this.x = x;
    this.y = y;
    this.vx = vx;
    this.vy = vy;
    this.r = r;
    alive = true;
  }
  
  public void create(int x,int y,int vx,int vy,int r,int c){
    this.x = x;
    this.y = y;
    this.vx = vx;
    this.vy = vy;
    this.r = r;
    this.c = c;
    alive = true;
  }
  
  public void update(){
    super.update();
    if(alive && x > width || x<0 || y > height || y<0){
      alive = !true;
    }
  }
  
  public void render(){
    fill(c,a);
    super.render();
  }
}
class BulletManager {
  ArrayList<Bullet> bullets;

  BulletManager() {
    bullets = new ArrayList<Bullet>();
    for (int i = 0; i<30; i++) {
      Bullet b = new Bullet(0, 0, 10, 255);
      b.alive = false;
      bullets.add(b);
    }
  }

  public void render() {
    for (Bullet b : bullets) {
      b.render();
      b.update();
    }
  }

  public void shoot(int x, int y, int vx, int vy, int r) {
    for (Bullet b : bullets) {
      if (!b.alive) {
        b.create(x, y, vx, vy, r);
        bulletSound.stop();
        bulletSound.play();
        return;
      }
    }
  }
  
  public void shoot(int x, int y, int vx, int vy, int r,int c) {
    for (Bullet b : bullets) {
      if (!b.alive) {
        b.create(x, y, vx, vy, r,c);
        bulletSound.stop();
        bulletSound.play();
        return;
      }
    }
  }

  public Bullet collision(Sprite e) {
    for (Bullet b : bullets) {
      if (!b.alive)
        continue;

      if ( dist(e.x, e.y, b.x, b.y)<= (b.r+e.r)/2) {
        return b;
      }
    }
    return null;
  }
  
  public Bullet collision(PowerUp e) {
    for (Bullet b : bullets) {
      if (!b.alive)
        continue;

      if ( dist(e.x, e.y, b.x, b.y)<= (b.r+e.r)/2) {
        Level.p.pustate = true;
        e.alive = !true;
        return b;
      }
    }
    return null;
  }
}
class Enemy extends Sprite {
  Trail trail;
  int type;

  int rColor;

  int hp = 3;
  int hitTime;
  boolean isHit;

  Enemy(int x, int y, int r, int c) {
    super(x, y, r, c);
    
    if(type == 1){
      this.c = 0xff0d0a8e;
    }
    if(type == 2){
      this.c = 0xff7a1d8a;
    }
    
    rColor = this.c;
    trail = new Trail(this);
    trail.vx = 3;
    t = (int)(random(100));
  }

  public void update() {
    super.update();
    t+=1;
    
    
    if(type == 0){
      y += (int)(sin(t/10f)*5);
    }
    if(type == 1){
      y = height/2+(int)(sin(t/75f)*(height/2-50));
    }
    if(type == 2){
      y += (int)(sin(t/10f)*5);
    }
    
    if (t % 5 == 0)trail.spawn(x, y);

    if (isHit) {
      if (t % 10 == 0) {
        if (c == rColor)
          c = 255;
        else
          c = rColor;
      }
      if (hitTime-- <= 0) {
        isHit = !true;
        c = rColor;
      }
    }
    
    if(alive && x<0){
      gsm.setState(new PlayState(currentDay));
    }
  }

  public void render() {
    trail.render();
    fill(c,a);
    super.render();
  }
  
  public void setType(int t){
    type = t;
    if(type == 0){
      c = 0xff1d0000;
      rColor = 0xff1d0000;
    }
    if(type == 1){
      c = 0xff0d0a8e;
      rColor = 0xff0d0a8e;
    }
    if(type == 2){
      c = 0xff13710b;
      rColor = 0xff13710b;
    }
    if(type == 3){
      c = 0xffcdc215;
      rColor = 0xffcdc215;
    }
  }
}
class EnemyManager {
  ArrayList<Enemy> enemies;

  EnemyManager() {
    enemies = new ArrayList<Enemy>();

    for (int i = 0; i<20; i++) {
      Enemy e = new Enemy(0, 0, 75, 0xff1d0000);
      e.alive = false;
      enemies.add(e);
    }
  }

  public void render() {
    for (Enemy e : enemies) {
      if (!e.alive)continue;
      e.update();
      e.render();
    }
  }

  public Enemy create(int x, int y) {
    for (Enemy e : enemies) {
        if (e.alive)continue;
        e.x = x;
        e.y = y;
        e.r = 75;
        e.alive = true;
        
        e.setType((int)random(0,4));
        e.vx = -2;
        e.hp = 5;
        if(currentDay == 0 ){
          e.type = 0;
        }
        if(currentDay == 1 ){
          if(e.type > 1){
            e.type = 1;
          }
        }
        
        if(currentDay == 2 ){
          if(e.type > 2){
            e.type -= 2;
          }
        }
        
        
        if(e.type == 2){
          e.vx = -1;
          e.hp = 10;
        }
        if(e.type == 3){
          e.vx = -4;
          e.hp = 1;
        }
        return e;
    }


    return null;
  }

  public Enemy collision(BulletManager b) {
    Enemy tmp = null;
    for (Enemy e : enemies) {
      if (!e.alive) {
        continue;
      }
      Bullet bb = b.collision(e);
      if (bb != null) {
        bb.alive = !true;
        e.hp--;
        e.isHit = true;
        e.hitTime = 30;
        particles.explode(e.x, e.y, 2,e.c);
        e.x += 50;
        if (e.hp <= 0) {
          e.alive = !true;
          particles.explode(e.x, e.y, 15,e.c);
          enemyKilled++;
          tmp = e;
        }
        break;
      }
    }

    return tmp;
  }

  public void gravitate(Player p) {
    for (Enemy b : enemies) {
      if (!b.alive)
        continue;
      float d = dist(b.x, b.y, p.x, p.y);
      if (d<500) {
        float a = atan2(p.x-b.x, p.y-b.y);
        b.x -= (int)(-sin(a)*3);
        b.y -= (int)(-cos(a)*3);
        if (d <= b.r+p.r/2) {
          p.alive = !true;
        }
      }
    }
  }
}
class Explosion extends Sprite {
  int maxRadius;
  
  Explosion(int x,int y,int r,int c){
    super(x,y,0,c);
    maxRadius = r;
  }
  
  public void update(){
    if(!alive)
      return;
    r++;
    if(r >= maxRadius){
      alive = false;
    }
  }
  
  public void render(){
    if(!alive)
      return;
  }
}
class FadeState extends TransitionState{
  
  float t;
  float targetTime = 60;
  
  FadeState(State s){
    super(s);
  }
  
  public void update(){
    t++;
  }
  
  public void render(){
    if(t < targetTime/2){
      prev.render();
      fill(0,map(t/targetTime/2,0,0.1f,0,255));
    }
    else{
      next.render();
      fill(0,map(t/targetTime/2,0,0.5f,255,0));
    }
      
    rect(0,0,width,height);
      
    if(t >= targetTime){
      gsm.setState(next);
    }
  }
  
  public void dispose(){
  
  }
}
class GSM {
  State state;
  
  GSM(){
    state = null;
  }
  
  public void setState(State s){
    if(s != null){
      s.dispose();
    }
    state = s;
  }
  
  public void update(){
    if(state == null)
      return;
    state.update();
  }
  
  public void render(){
    if(state == null)
      return;
    state.render();
  }
  
  public State getState(){
    return state;
  }
}
class GameOver extends State {
  GameOver(){
    
  }
  public void update(){}
  public void render(){
    background(0);
    fill(255);
    textFont(font);
    text("Thank you!",width/2,height/2);
    textFont(font2);
    text("If you liked our game: \nplease help make a difference in someone's life!\nDonate to:\nwww.cancer.org \nwww.cancersurvivorsfund.com \nwww.childrencancer.org \nOr any charities.",width/2,height/2);
  }
  public void dispose(){}
}
static class Level {
  static Player p;
  static int[][] enemies = {{10,0,0,0},{10,0,0,0},{10,0,0,0},{10,0,0,0},{10,0,0,0},{10,0,0,0},{10,0,0,0}};
  static String[] strings = {"Mommy? Where are we? \nDoctor (to parents): It's safe to assume that you're quite familiar with such words by now, but I'm afraid her situation is critical. \nI just want you to know that we're doing our best to help her go through this, we're already carrying out our most desperate measures, \n but I'm afraid we don't have much time. \nFather: We understand, thank you for your efforts doctor. \n[Doctors vaccinate the little girl] \n",
                             "Mother: Hey honey, how are you holding up? \nDaughter:*Coughs* I don't feel too well mommy. \n",
                             "Daughter: Mommy, when I get out of here, will you please let me get a bike? I want to play with my cousins! \nMother: Of course honey, I will get you anything you want! \nDaughter: When will I be able to leave this room? \nMother: Soon baby, soon. \n",
                             "[Birthday Party] \nMother: *Forcing a smile* It's your ninth birthday! Make a wish! \nDaughter: *Mumbles* I wish I could leave this place. \n[The room goes into deep silence] \n",
                             "Daughter: Mommy? Why aren't flowers making me happy anymore? \nMother: Flowers help remind us to stay positive and cheerful during tough times, \n even when life takes away our most precious treasures. \nYou shouldn't be feeling like that darling, \n you still have your whole family here with you \n and you will soon leavethis room and go back to your sweet old self again. \nJust be patient, and stay strong, just like those flowers, \n they still stand tall even after being taken out of their dirt. \nDaughter: But mom, flowers always end up dying. *coughs* \n[Mother bursts into tears]",
                             "Daughter: Mommy, do you still think I'm pretty? \nMother: Of course sweetie! You are and always will be my precious diamond. \nDaughter: I think I looked better with my hair. I miss it. \n",
                             "Father: *Shouting* What do you mean \"our best\"? \n You gave us hope! \n You said that your new cure will get my baby back on her feet! You are all thieves! \nI am going to sue you, all of you! \n"
                            };
  static int[] currentEnemies = {0,0,0,0};
}
class Particle extends Sprite {
  Particle(int x, int y, int r, int c) {
    super(x, y, r, c);
    alive = !true;
  }

  public void emit(int x, int y, int vx, int vy, int r) {
    this.x = x;
    this.y = y;
    this.vx = vx;
    this.vy = vy;
    this.r = r;
    a = 100;
    alive = true;
  }

  public void emit(int x, int y, int vx, int vy, int r, int c) {
    this.x = x;
    this.y = y;
    this.vx = vx;
    this.vy = vy;
    this.r = r;
    this.c = c;
    a = 100;
    alive = true;
  }

  public void update() {
    super.update();
    a -= 1;
    if (alive && a <=0 || x > width || x<0 || y > height || y<0) {
      alive = !true;
    }
  }

  public void render() {
    fill(c, a);
    super.render();
  }
}
class ParticleManager {
  ArrayList<Particle> particles;
  
  ParticleManager(){
    particles = new ArrayList<Particle>();
    
    for(int i = 0; i<100; i++){
      Particle p = new Particle(0,0,10,255);
      p.alive = false;
      particles.add(p);
    }
  }
  
  public void render(){
    for(Particle p : particles){
      p.render();
      p.update();
    }
  }
  
  public void emit(int x,int y, int vx , int vy,int r){
    for(Particle p : particles){
      if(!p.alive){
        p.emit(x,y,vx,vy,r);
        return;
      }
    }
  }
  
  public void emit(int x,int y, int vx , int vy,int r,int c){
    for(Particle p : particles){
      if(!p.alive){
        p.emit(x,y,vx,vy,r,c);
        return;
      }
    }
  }
  
  public void explode(int x,int y,int intensity){
    for(int i=0; i< intensity; i++){
      emit(x,y,(int)random(-5,5),(int)random(-5,5),20);
    }
  }
  
  public void explode(int x,int y,int intensity,int c){
    for(int i=0; i< intensity; i++){
      emit(x,y,(int)random(-5,5),(int)random(-5,5),20,c);
    }
  }
}
class PlayState extends State{
  Player s;
  //PowerUp pu;
  
  EnemyManager enemies;
  
  int wavecount = 1;
  int strCount = 1;
  int strRate = 10;
  
  int count = 60*4;
  
  int t=1;
  int bulletTimer;
  int rate = 5;
  float ba;
  int day ;
  
  int enemySize = 10+currentDay*5;
  int enemyCreated;
  
  PlayState(int day){
    isZooming = true;
    s = new Player(100,height/2,50,255);
    Level.p = s;
    //pu = new PowerUp(200,200,30,0);
    enemies = new EnemyManager();
    this.day = day;
    enemyKilled = 0;
    Level.currentEnemies = new int[] {0,0,0,0};
  }
   public void corup (){
    if(mousePressed){
      ba = atan2(s.x-mouseX,s.y-mouseY);
      //System.out.println(ba);
      if(t % (s.pustate ? 2 : rate) == 0){
        bullets.shoot(s.x+(int)(-sin(ba)*s.r/2),s.y+(int)(-cos(ba)*s.r/2),(int)(-sin(ba)*10),(int)(-cos(ba+random(0.1f))*14),20,s.pustate ? s.hue : s.c);
        particles.explode(s.x+(int)(-sin(ba)*s.r/2),s.y+(int)(-cos(ba)*s.r/2),1);
      }
      //bulletTimer++;
    }
    
    Enemy e = enemies.collision(bullets);
    if(e != null){
      particles.explode(e.x,e.y,10);
      isShaking = true;
      shakeTimer = 5;
      shakeIntensity = 100;
    }
    
    if(count < 60 && t % 50 == 0){
      if(enemyCreated < enemySize){
        enemies.create(1280+100,(int)random(100,600));
        enemyCreated++;
      }
    }
    
    if(keyPressed){
      if(key == 'u' || key == 'U'){
        s.move(0,-3);
      }else if(key == 'j' || key == 'J'){
        s.move(0,3);
      }
    }
   }
   
  public void update(){
   
    corup();
    if(enemyKilled >= enemySize){
      currentDay++;
      if(currentDay == 7){
        gsm.setState(new FadeState(new GameOver()));
      }else
        gsm.setState(new FadeState(new TextState(currentDay)));
    }
    //bullets.collision(pu);
    //enemies.gravitate(s);
  }
  
  public void render(){
    background(0,201,58+20+sin(t++/100f)*30);
    image(light,100,100);
    image(light,600,300);
    
    image(light,s.x-light.width/2,s.y-light.height/2);
    s.render();
    enemies.render();
    
    //pu.render();
    
    fill(255);
    if(count-- >= 60){
      textFont(font);
      text("WAVE: "+ day,width/2,height/2-20);
      text(round(count/60),width/2,height/2+40);
      
      /*fill(255);
      textFont(font2);
      text(str3,width/2,height - 300);
      if(t % strRate == 0 && strCount < str2.length()){
        strCount++;
        str3 = str2.substring(0,strCount);
      }*/
    }else{
      fill(200);
      textFont(font2);
      text("WAVE: "+ day,200,50);
      text("Shoot The Enemy !!!",width/2,height - 50);
    }
    
  }
  
  public void dispose(){
    
  }
}
class Player extends Sprite{
  Trail trail;
  int hue;
  
  boolean pustate = !true;
  
  Player(int x,int y,int r,int c){
    super(x,y,r,c);
    trail = new Trail(this);
    trail.vx = -3;
    x = 50;
  }
  
  public void update(){
    t++;
    y += (int)(sin(t/10f)*2);
    if(t % 5 == 0)trail.spawn(x,y);
  }
  
  public void render(){
    /*beginShape();
    vertex(x,y-r/2);
    vertex(x-30,y);
    vertex(x,y+r/2);
    endShape(CLOSE);*/
    trail.render();
    
    if(pustate){
      fill(map(sin(t/10f),-1,1,0,255),200,200);
      hue = g.fillColor;
    }else{
      fill(c,a);
    }
    super.render();
    fill(255);
  }
  
  public void move(int x,int y){
    this.x += x;
    this.y += y;
  }

}
class PowerUp extends Sprite {
  int t;
  
  PowerUp(int x,int y,int r,int c){
    super(x,y,r,c);
  }
  
  public void update(){
    super.update();
  }
  
  public void render(){
    fill(map(sin(t++/10f),-1,1,0,255),200,200);
    super.render();
  }
}
class SplashState extends State {
  
  int t=1;
  PImage splash;
  
  SplashState(){
    splash = loadImage("logo.jpg");
    isZooming = !true;
  }
  
  public void update(){
    if(t++ >= 300){
      gsm.setState(new FadeState(new TextState(0))); 
    }
  }
  
  public void render(){
    background(0,201,58+20+sin(t++/100f)*30);
    image(splash,0,0,1280,720);
  }
  
  public void dispose(){
  
  }
}
class Sprite{
  
  int x,y,vx,vy;
  int r;
  int c;
  int a;
  
  int t;
  boolean original = true;
  
  boolean alive = true;
  
  Sprite(int x,int y,int r,int c){
    this.x = x;
    this.y = y;
    this.r = r;
    this.c = c;
    a = 255;
  }
  
  public void update(){
    x += vx;
    y += vy;
  }
  
  public void render(){
    if(alive){
      //fill(c,a);
      ellipse(x,y,r,r);
      update();
    }
  }
}
abstract class State{
  public abstract void update();
  public abstract void render();
  public abstract void dispose();
}
class StoryState extends State {
  String title;
  int t;

  StoryState() {
  }

  public void update() {
    if (t++ >= 60) {
      gsm.setState(new FadeState(new PlayState(currentDay+1)));
    }
  }
  public void render() {
    background(0);
    fill(255);
    text("DAY "+(currentDay+1), width/2, height/2);
  }
  public void dispose() {
  }
}
class TextState extends State{
  PImage img;
  
  int t = 1;
  
  String str = "Mom here are e ?\nagdagdagadgdagdagad\nagdagdagdagdagda\ndagdagdagdag\n"; 
  String str2 = "";
  int strCount = 1;
  int strRate = 2;
  
  TextState(int s){
    str = Level.strings[s];
    img = loadImage("hospital.jpg");
  }
  public void update(){
    t++;
    if(strCount >= str.length()-1 || keyPressed){
      gsm.setState(new FadeState(new StoryState()));
    }
  }
  public void render(){
    image(img,0,0,width,height);
    fill(255);
    textFont(font2);
    text(str2,width/2,height - 300);
    if(t % strRate == 0 && strCount < str.length()){
      strCount++;
      if(str.charAt(strCount) == '\n'){
        str = str.substring(strCount+1);
        strCount = 0;
      }
      str2 = str.substring(0,strCount);
    }
  }
  public void dispose(){}
}
class Trail {
  Sprite s;
  ArrayList<Sprite> trails;
  int radius;
  int t;
  int vx;

  Trail(Sprite s) {
    this.s = s;
    radius = 30;
    trails = new ArrayList<Sprite>();
    for (int i = 0; i<30; i++) {
      Sprite ss = new Sprite(0, 0, s.r, s.c);
      ss.alive = false;
      trails.add(ss);
    }
  }

  public void spawn(int x, int y) {
    for (Sprite ss : trails) {
      if (ss.alive == false) {
        ss.x = x;
        ss.y = (int)(y+sin(t/10f)*4);
        ss.a = 200;
        ss.r = ceil(radius/0.8f);
        ss.alive = true;
        return;
      }
    }
  }

  public void render() {
    for (int i = 0; i<trails.size(); i++) {
      Sprite ss = trails.get(i);
      if (ss.alive) {
        fill(ss.c,ss.a);
        ss.render();
        ss.a-=5;
        ss.r-=1;
        ss.x+=vx;
        if (ss.a <= 0 || ss.r <= 0) {
          ss.alive = false;
        }
      }
    }

    t++;
  }
}
abstract class TransitionState extends State{
  State prev;
  State next;
  
  TransitionState(State s){
    prev = gsm.getState();
    next = s;
  }
}
  public void settings() {  size(1280, 720, P2D); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "prote" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
